@@ -916,94 +916,95 @@ static void help_update_terrain(const struct help_item *pitem,
 				char *title, int i)
 {
   char *buf = &long_buffer[0];
+  struct tile_type *ptype = get_tile_type(i);
 
   create_help_page(HELP_TERRAIN);
 
   if (i < T_COUNT) {
     sprintf(buf, "%d/%d.%d",
-	    tile_types[i].movement_cost,
-	    (int)(tile_types[i].defense_bonus/10),
-	    tile_types[i].defense_bonus%10);
+	    ptype->movement_cost,
+	    (int)(ptype->defense_bonus/10),
+	    ptype->defense_bonus%10);
     gtk_set_label(help_tlabel[0][1], buf);
 
     sprintf(buf, "%d/%d/%d",
-	    tile_types[i].food,
-	    tile_types[i].shield,
-	    tile_types[i].trade);
+	    ptype->food,
+	    ptype->shield,
+	    ptype->trade);
     gtk_set_label(help_tlabel[0][4], buf);
 
-    if (*(tile_types[i].special_1_name)) {
+    if (*(ptype->special_1_name)) {
       sprintf(buf, _("%s F/R/T:"),
-	      tile_types[i].special_1_name);
+	      ptype->special_1_name);
       gtk_set_label(help_tlabel[1][0], buf);
       sprintf(buf, "%d/%d/%d",
-	      tile_types[i].food_special_1,
-	      tile_types[i].shield_special_1,
-	      tile_types[i].trade_special_1);
+	      ptype->food_special_1,
+	      ptype->shield_special_1,
+	      ptype->trade_special_1);
       gtk_set_label(help_tlabel[1][1], buf);
     } else {
       gtk_set_label(help_tlabel[1][0], "");
       gtk_set_label(help_tlabel[1][1], "");
     }
 
-    if (*(tile_types[i].special_2_name)) {
+    if (*(ptype->special_2_name)) {
       sprintf(buf, _("%s F/R/T:"),
-	      tile_types[i].special_2_name);
+	      ptype->special_2_name);
       gtk_set_label(help_tlabel[1][3], buf);
       sprintf(buf, "%d/%d/%d",
-	      tile_types[i].food_special_2,
-	      tile_types[i].shield_special_2,
-	      tile_types[i].trade_special_2);
+	      ptype->food_special_2,
+	      ptype->shield_special_2,
+	      ptype->trade_special_2);
       gtk_set_label(help_tlabel[1][4], buf);
     } else {
       gtk_set_label(help_tlabel[1][3], "");
       gtk_set_label(help_tlabel[1][4], "");
     }
 
-    if (tile_types[i].road_trade_incr > 0) {
+    if (ptype->road_trade_incr > 0) {
       sprintf(buf, _("+%d Trade / %d"),
-	      tile_types[i].road_trade_incr,
-	      tile_types[i].road_time);
-    } else if (tile_types[i].road_time > 0) {
+	      ptype->road_trade_incr,
+	      ptype->road_time);
+    } else if (ptype->road_time > 0) {
       sprintf(buf, _("no extra / %d"),
-	      tile_types[i].road_time);
+	      ptype->road_time);
     } else {
       strcpy(buf, _("n/a"));
     }
     gtk_set_label(help_tlabel[2][1], buf);
 
     strcpy(buf, _("n/a"));
-    if (tile_types[i].irrigation_result == i) {
-      if (tile_types[i].irrigation_food_incr > 0) {
+    if (ptype->irrigation_result == i) {
+      if (ptype->irrigation_food_incr > 0) {
 	sprintf(buf, _("+%d Food / %d"),
-		tile_types[i].irrigation_food_incr,
-		tile_types[i].irrigation_time);
+		ptype->irrigation_food_incr,
+		ptype->irrigation_time);
       }
-    } else if (tile_types[i].irrigation_result != T_NONE) {
+    } else if (ptype->irrigation_result != T_NONE) {
       sprintf(buf, "%s / %d",
-	      tile_types[tile_types[i].irrigation_result].terrain_name,
-	      tile_types[i].irrigation_time);
+	      get_tile_type(ptype->irrigation_result)->terrain_name,
+	      ptype->irrigation_time);
     }
     gtk_set_label(help_tlabel[2][4], buf);
 
     strcpy(buf, _("n/a"));
-    if (tile_types[i].mining_result == i) {
-      if (tile_types[i].mining_shield_incr > 0) {
+    if (ptype->mining_result == i) {
+      if (ptype->mining_shield_incr > 0) {
 	sprintf(buf, _("+%d Res. / %d"),
-		tile_types[i].mining_shield_incr,
-		tile_types[i].mining_time);
+		ptype->mining_shield_incr,
+		ptype->mining_time);
       }
-    } else if (tile_types[i].mining_result != T_NONE) {
+    } else if (ptype->mining_result != T_NONE) {
       sprintf(buf, "%s / %d",
-	      tile_types[tile_types[i].mining_result].terrain_name,
-	      tile_types[i].mining_time);
+	      get_tile_type(ptype->mining_result)->terrain_name,
+	      ptype->mining_time);
     }
     gtk_set_label(help_tlabel[3][1], buf);
 
-    if (tile_types[i].transform_result != T_NONE) {
+    if (ptype->transform_result != T_NONE) {
       sprintf(buf, "%s / %d",
-	      tile_types[tile_types[i].transform_result].terrain_name,
-	      tile_types[i].transform_time);
+	      get_tile_type(ptype->transform_result)->terrain_name,
+	      ptype->transform_time);
     } else {
       strcpy(buf, "n/a");
     }
