@@ -10,6 +10,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -22,27 +23,30 @@
 
 #include "astring.h"
 #include "fcintl.h"
+#include "government.h"
 #include "log.h"
 #include "map.h"
 #include "mem.h"
 #include "support.h"
 #include "unit.h"
 
-#include "chatline.h"
-#include "cityrep.h"
 #include "civclient.h"
 #include "clinet.h"
 #include "control.h"
+#include "options.h"
+#include "packhand.h"
+
+#include "chatline.h"
+#include "cityrep.h"
 #include "dialogs.h"
 #include "finddlg.h"
 #include "gotodlg.h"
 #include "gui_stuff.h"
 #include "helpdlg.h"
-#include "mapctrl.h"   /* center_on_unit */
+#include "mapctrl.h"		/* center_on_unit */
 #include "messagedlg.h"
 #include "messagewin.h"
 #include "optiondlg.h"
-#include "options.h"
 #include "plrdlg.h"
 #include "ratesdlg.h"
 #include "repodlgs.h"
@@ -72,13 +76,15 @@ enum MenuID {
   MENU_GAME_DISCONNECT,
   MENU_GAME_QUIT,
   
-  MENU_KINGDOM_TAX_RATE,
-  MENU_KINGDOM_FIND_CITY,
-  MENU_KINGDOM_WORKLISTS,
-  MENU_KINGDOM_REVOLUTION,
+  MENU_GOVERNMENT_TAX_RATE,
+  MENU_GOVERNMENT_FIND_CITY,
+  MENU_GOVERNMENT_WORKLISTS,
+  MENU_GOVERNMENT_REVOLUTION,
 
   MENU_VIEW_SHOW_MAP_GRID,
+  MENU_VIEW_SHOW_NATIONAL_BORDERS,
   MENU_VIEW_SHOW_CITY_NAMES,
+  MENU_VIEW_SHOW_CITY_GROWTH_TURNS,
   MENU_VIEW_SHOW_CITY_PRODUCTIONS,
   MENU_VIEW_SHOW_TERRAIN,
   MENU_VIEW_SHOW_COASTLINE,
@@ -106,14 +112,19 @@ enum MenuID {
   MENU_ORDER_SENTRY,
   MENU_ORDER_PILLAGE,
   MENU_ORDER_HOMECITY,
+  MENU_ORDER_UNLOAD_TRANSPORTER,
+  MENU_ORDER_LOAD,
   MENU_ORDER_UNLOAD,
   MENU_ORDER_WAKEUP_OTHERS,
   MENU_ORDER_AUTO_SETTLER,   /* shared with AUTO_ATTACK */
   MENU_ORDER_AUTO_EXPLORE,
-  MENU_ORDER_CONNECT,
+  MENU_ORDER_CONNECT_ROAD,
+  MENU_ORDER_CONNECT_RAIL,
+  MENU_ORDER_CONNECT_IRRIGATE,
   MENU_ORDER_PATROL,
   MENU_ORDER_GOTO,
   MENU_ORDER_GOTO_CITY,
+  MENU_ORDER_RETURN,
   MENU_ORDER_DISBAND,
   MENU_ORDER_DIPLOMAT_DLG,
   MENU_ORDER_NUKE,
@@ -193,20 +204,20 @@ static void game_menu_callback(gpointer callback_data,
 /****************************************************************
 ...
 *****************************************************************/
-static void kingdom_menu_callback(gpointer callback_data,
+static void government_menu_callback(gpointer callback_data,
 				  guint callback_action, GtkWidget *widget)
 {
   switch(callback_action) {
-  case MENU_KINGDOM_TAX_RATE:
+  case MENU_GOVERNMENT_TAX_RATE:
     popup_rates_dialog();
     break;
-  case MENU_KINGDOM_FIND_CITY:
+  case MENU_GOVERNMENT_FIND_CITY:
     popup_find_dialog();
     break;
-  case MENU_KINGDOM_WORKLISTS:
+  case MENU_GOVERNMENT_WORKLISTS:
     popup_worklists_report(game.player_ptr);
     break;
-  case MENU_KINGDOM_REVOLUTION:
+  case MENU_GOVERNMENT_REVOLUTION:
     popup_revolution_dialog();
     break;
   }
@@ -225,9 +236,20 @@ static void view_menu_callback(gpointer callback_data, guint callback_action,
     if (draw_map_grid ^ GTK_CHECK_MENU_ITEM(widget)->active)
       key_map_grid_toggle();
     break;
+  case MENU_VIEW_SHOW_NATIONAL_BORDERS:
+    if (draw_borders ^ GTK_CHECK_MENU_ITEM(widget)->active) {
+      key_map_borders_toggle();
+    }
+    break;
   case MENU_VIEW_SHOW_CITY_NAMES:
-    if (draw_city_names ^ GTK_CHECK_MENU_ITEM(widget)->active)
+    if (draw_city_names ^ GTK_CHECK_MENU_ITEM(widget)->active) {
       key_city_names_toggle();
+      menus_set_sensitive("<main>/_View/City G_rowth", draw_city_names);
+    }
+    break;
+  case MENU_VIEW_SHOW_CITY_GROWTH_TURNS:
+    if (draw_city_growth ^ GTK_CHECK_MENU_ITEM(widget)->active)
+      key_city_growth_toggle();
     break;
   case MENU_VIEW_SHOW_CITY_PRODUCTIONS:
     if (draw_city_productions ^ GTK_CHECK_MENU_ITEM(widget)->active)
@@ -359,8 +381,14 @@ static void orders_menu_callback(gpointer callback_data,
    case MENU_ORDER_HOMECITY:
     key_unit_homecity();
     break;
-   case MENU_ORDER_UNLOAD:
-    key_unit_unload();
+  case MENU_ORDER_UNLOAD_TRANSPORTER:
+    key_unit_unload_all();
+    break;
+  case MENU_ORDER_LOAD:
+    request_unit_load(get_unit_in_focus(), NULL);
+    break;
+  case MENU_ORDER_UNLOAD:
+    request_unit_unload(get_unit_in_focus());
     break;
    case MENU_ORDER_WAKEUP_OTHERS:
     key_unit_wakeup_others();
@@ -372,8 +400,14 @@ static void orders_menu_callback(gpointer callback_data,
    case MENU_ORDER_AUTO_EXPLORE:
     key_unit_auto_explore();
     break;
-   case MENU_ORDER_CONNECT:
-    key_unit_connect();
+   case MENU_ORDER_CONNECT_ROAD:
+    key_unit_connect(ACTIVITY_ROAD);
+    break;
+   case MENU_ORDER_CONNECT_RAIL:
+    key_unit_connect(ACTIVITY_RAILROAD);
+    break;
+   case MENU_ORDER_CONNECT_IRRIGATE:
+    key_unit_connect(ACTIVITY_IRRIGATE);
     break;
    case MENU_ORDER_PATROL:
     key_unit_patrol();
@@ -385,6 +419,11 @@ static void orders_menu_callback(gpointer callback_data,
     if(get_unit_in_focus())
       popup_goto_dialog();
     break;
+   case MENU_ORDER_RETURN:
+    if (get_unit_in_focus()) {
+      request_unit_return(get_unit_in_focus());
+    }
+    break;
    case MENU_ORDER_DISBAND:
     key_unit_disband();
     break;
@@ -544,234 +583,254 @@ static GtkItemFactoryEntry menu_items[]	=
   /* Game menu ... */
   { "/" N_("_Game"),					NULL,
 	NULL,			0,					"<Branch>"	},
-  { "/" N_("Game") "/tearoff1",				NULL,
+  { "/" N_("_Game") "/tearoff1",			NULL,
 	NULL,			0,					"<Tearoff>"	},
-  { "/" N_("Game") "/sep1",				NULL,
+  { "/" N_("_Game") "/sep1",				NULL,
 	NULL,			0,					"<Separator>"	},
-  { "/" N_("Game") "/" N_("_Local Options"),		NULL,
+  { "/" N_("_Game") "/" N_("_Local Options"),		NULL,
 	game_menu_callback,	MENU_GAME_OPTIONS					},
-  { "/" N_("Game") "/" N_("Messa_ge Options"),		NULL,
+  { "/" N_("_Game") "/" N_("Messa_ge Options"),		NULL,
 	game_menu_callback,	MENU_GAME_MSG_OPTIONS					},
-  { "/" N_("Game") "/" N_("_Save Settings"),		NULL,
+  { "/" N_("_Game") "/" N_("_Save Settings"),		NULL,
 	game_menu_callback,	MENU_GAME_SAVE_SETTINGS					},
-  { "/" N_("Game") "/sep2",				NULL,
+  { "/" N_("_Game") "/sep2",				NULL,
 	NULL,			0,					"<Separator>"	},
-  { "/" N_("Game") "/" N_("Server Opt _initial"),	NULL,
+  { "/" N_("_Game") "/" N_("Server Opt _initial"),	NULL,
 	game_menu_callback,	MENU_GAME_SERVER_OPTIONS1				},
-  { "/" N_("Game") "/" N_("Server Opt _ongoing"),	NULL,
+  { "/" N_("_Game") "/" N_("Server Opt _ongoing"),	NULL,
 	game_menu_callback,	MENU_GAME_SERVER_OPTIONS2				},
-  { "/" N_("Game") "/sep3",				NULL,
+  { "/" N_("_Game") "/sep3",				NULL,
 	NULL,			0,					"<Separator>"	},
-  { "/" N_("Game") "/" N_("_Export Log"),		NULL,
+  { "/" N_("_Game") "/" N_("_Export Log"),		NULL,
 	game_menu_callback,	MENU_GAME_OUTPUT_LOG					},
-  { "/" N_("Game") "/" N_("_Clear Log"),		NULL,
+  { "/" N_("_Game") "/" N_("_Clear Log"),		NULL,
 	game_menu_callback,	MENU_GAME_CLEAR_OUTPUT					},
-  { "/" N_("Game") "/sep4",				NULL,
+  { "/" N_("_Game") "/sep4",				NULL,
 	NULL,			0,					"<Separator>"	},
-  { "/" N_("Game") "/" N_("_Disconnect"),		NULL,
+  { "/" N_("_Game") "/" N_("_Disconnect"),		NULL,
 	game_menu_callback,	MENU_GAME_DISCONNECT					},
-  { "/" N_("Game") "/" N_("_Quit"),			"<control>q",
+  { "/" N_("_Game") "/" N_("_Quit"),			"<control>q",
 	gtk_main_quit,		0							},
-  /* Kingdom menu ... */
-  { "/" N_("_Kingdom"),					NULL,
+  /* Government menu ... */
+  { "/" N_("Gov_ernment"),				NULL,
 	NULL,			0,					"<Branch>"	},
-  { "/" N_("Kingdom") "/tearoff1",			NULL,
+  { "/" N_("Gov_ernment") "/tearoff1",			NULL,
 	NULL,			0,					"<Tearoff>"	},
-  { "/" N_("Kingdom") "/" N_("_Tax Rates"),		"<shift>t",
-	kingdom_menu_callback,	MENU_KINGDOM_TAX_RATE					},
-  { "/" N_("Kingdom") "/sep1",				NULL,
+  { "/" N_("Gov_ernment") "/" N_("_Tax Rates"),		"<shift>t",
+	government_menu_callback,MENU_GOVERNMENT_TAX_RATE				},
+  { "/" N_("Gov_ernment") "/sep1",			NULL,
 	NULL,			0,					"<Separator>"	},
-  { "/" N_("Kingdom") "/" N_("_Find City"),		"<shift>f",
-	kingdom_menu_callback,	MENU_KINGDOM_FIND_CITY					},
-  { "/" N_("Kingdom") "/" N_("Work_lists"),		"<shift>l",
-	kingdom_menu_callback,	MENU_KINGDOM_WORKLISTS					},
-  { "/" N_("Kingdom") "/sep2",				NULL,
+  { "/" N_("Gov_ernment") "/" N_("_Find City"),		"<shift>f",
+	government_menu_callback,MENU_GOVERNMENT_FIND_CITY				},
+  { "/" N_("Gov_ernment") "/" N_("Work_lists"),		"<shift>l",
+	government_menu_callback,MENU_GOVERNMENT_WORKLISTS				},
+  { "/" N_("Gov_ernment") "/sep2",			NULL,
 	NULL,			0,					"<Separator>"	},
-  { "/" N_("Kingdom") "/" N_("_Revolution"),	        "<shift>r",
-	kingdom_menu_callback,	MENU_KINGDOM_REVOLUTION					},
+  { "/" N_("Gov_ernment") "/" N_("_Change Government"),	NULL,
+	NULL,           0,                  "<Branch>"    },
+  { "/" N_("Gov_ernment") "/" N_("_Change Government") "/" N_("_Revolution!"),
+                                                 "<shift>r",
+	government_menu_callback,	MENU_GOVERNMENT_REVOLUTION     },
+  { "/" N_("Gov_ernment") "/" N_("_Change Government") "/sep1", NULL,
+	NULL,			0,			"<Separator>"	},
   /* View menu ... */
   { "/" N_("_View"),					NULL,
 	NULL,			0,					"<Branch>"	},
-  { "/" N_("View") "/tearoff1",				NULL,
+  { "/" N_("_View") "/tearoff1",			NULL,
 	NULL,			0,					"<Tearoff>"	},
-  { "/" N_("View") "/" N_("Map _Grid"),			"<control>g",
+  { "/" N_("_View") "/" N_("Map _Grid"),			"<control>g",
 	view_menu_callback,	MENU_VIEW_SHOW_MAP_GRID,		"<CheckItem>"	},
-  { "/" N_("View") "/" N_("City _Names"),		"<control>n",
+  { "/" N_("_View") "/" N_("National _Borders"),		"<control>b",
+	view_menu_callback,	MENU_VIEW_SHOW_NATIONAL_BORDERS,	"<CheckItem>"	},
+  { "/" N_("_View") "/" N_("City _Names"),		"<control>n",
 	view_menu_callback,	MENU_VIEW_SHOW_CITY_NAMES,		"<CheckItem>"	},
-  { "/" N_("View") "/" N_("City _Productions"),		"<control>p",
+  { "/" N_("_View") "/" N_("City G_rowth"),		"<control>r",
+	view_menu_callback,	MENU_VIEW_SHOW_CITY_GROWTH_TURNS,
+	"<CheckItem>"	},
+  { "/" N_("_View") "/" N_("City _Productions"),		"<control>p",
 	view_menu_callback,	MENU_VIEW_SHOW_CITY_PRODUCTIONS,	"<CheckItem>"	},
-  { "/" N_("View") "/sep1",				NULL,
+  { "/" N_("_View") "/sep1",				NULL,
 	NULL,			0,					"<Separator>"	},
-  { "/" N_("View") "/" N_("Terrain"),                   NULL,
+  { "/" N_("_View") "/" N_("Terrain"),			NULL,
         view_menu_callback,     MENU_VIEW_SHOW_TERRAIN,	                "<CheckItem>"   },
-  { "/" N_("View") "/" N_("Coastline"),	                NULL,
+  { "/" N_("_View") "/" N_("Coastline"),		NULL,
         view_menu_callback,     MENU_VIEW_SHOW_COASTLINE,       	"<CheckItem>"   },
-  { "/" N_("View") "/" N_("Improvements"),		NULL,
+  { "/" N_("_View") "/" N_("Improvements"),		NULL,
 	NULL,			0,					"<Branch>"	},
-  { "/" N_("View") "/" N_("Improvements") "/tearoff1",	NULL,
+  { "/" N_("_View") "/" N_("Improvements") "/tearoff1",	NULL,
 	NULL,			0,					"<Tearoff>"	},
-  { "/" N_("View") "/" N_("Improvements") "/" N_("Roads & Rails"), NULL,
+  { "/" N_("_View") "/" N_("Improvements") "/" N_("Roads & Rails"), NULL,
 	view_menu_callback,	MENU_VIEW_SHOW_ROADS_RAILS,		"<CheckItem>"	},
-  { "/" N_("View") "/" N_("Improvements") "/" N_("Irrigation"), NULL,
+  { "/" N_("_View") "/" N_("Improvements") "/" N_("Irrigation"), NULL,
 	view_menu_callback,	MENU_VIEW_SHOW_IRRIGATION,		"<CheckItem>"	},
-  { "/" N_("View") "/" N_("Improvements") "/" N_("Mines"),	NULL,
+  { "/" N_("_View") "/" N_("Improvements") "/" N_("Mines"),	NULL,
 	view_menu_callback,	MENU_VIEW_SHOW_MINES,			"<CheckItem>"	},
-  { "/" N_("View") "/" N_("Improvements") "/" N_("Fortress & Airbase"), NULL,
+  { "/" N_("_View") "/" N_("Improvements") "/" N_("Fortress & Airbase"), NULL,
 	view_menu_callback,	MENU_VIEW_SHOW_FORTRESS_AIRBASE,	"<CheckItem>"	},
-  { "/" N_("View") "/" N_("Specials"),			NULL,
+  { "/" N_("_View") "/" N_("Specials"),			NULL,
 	view_menu_callback,	MENU_VIEW_SHOW_SPECIALS,		"<CheckItem>"	},
-  { "/" N_("View") "/" N_("Pollution & Fallout"),	NULL,
+  { "/" N_("_View") "/" N_("Pollution & Fallout"),	NULL,
 	view_menu_callback,	MENU_VIEW_SHOW_POLLUTION,		"<CheckItem>"	},
-  { "/" N_("View") "/" N_("Cities"),			NULL,
+  { "/" N_("_View") "/" N_("Cities"),			NULL,
 	view_menu_callback,	MENU_VIEW_SHOW_CITIES,			"<CheckItem>"	},
-  { "/" N_("View") "/" N_("Units"),			NULL,
+  { "/" N_("_View") "/" N_("Units"),			NULL,
 	view_menu_callback,	MENU_VIEW_SHOW_UNITS,			"<CheckItem>"	},
-  { "/" N_("View") "/" N_("Focus Unit"),		NULL,
+  { "/" N_("_View") "/" N_("Focus Unit"),		NULL,
 	view_menu_callback,	MENU_VIEW_SHOW_FOCUS_UNIT,		"<CheckItem>"	},
-  { "/" N_("View") "/" N_("Fog of War"),		NULL,
+  { "/" N_("_View") "/" N_("Fog of War"),		NULL,
 	view_menu_callback,	MENU_VIEW_SHOW_FOG_OF_WAR,		"<CheckItem>"	},
-  { "/" N_("View") "/sep2",				NULL,
+  { "/" N_("_View") "/sep2",				NULL,
 	NULL,			0,					"<Separator>"	},
-  { "/" N_("View") "/" N_("_Center View"),		"c",
+  { "/" N_("_View") "/" N_("_Center View"),		"c",
 	view_menu_callback,	MENU_VIEW_CENTER_VIEW					},
   /* Orders menu ... */
   { "/" N_("_Orders"),					NULL,
 	NULL,			0,					"<Branch>"	},
-  { "/" N_("Orders") "/tearoff1",			NULL,
+  { "/" N_("_Orders") "/tearoff1",			NULL,
 	NULL,			0,					"<Tearoff>"	},
-  { "/" N_("Orders") "/" N_("_Build City"),		"b",
+  { "/" N_("_Orders") "/" N_("_Build City"),		"b",
 	orders_menu_callback,	MENU_ORDER_BUILD_CITY					},
-  { "/" N_("Orders") "/" N_("Build _Road"),		"r",
+  { "/" N_("_Orders") "/" N_("Build _Road"),		"r",
 	orders_menu_callback,	MENU_ORDER_ROAD						},
-  { "/" N_("Orders") "/" N_("Build _Irrigation"),	"i",
+  { "/" N_("_Orders") "/" N_("Build _Irrigation"),	"i",
 	orders_menu_callback,	MENU_ORDER_IRRIGATE					},
-  { "/" N_("Orders") "/" N_("Build _Mine"),		"m",
+  { "/" N_("_Orders") "/" N_("Build _Mine"),		"m",
 	orders_menu_callback,	MENU_ORDER_MINE						},
-  { "/" N_("Orders") "/" N_("Transf_orm Terrain"),	"o",
+  { "/" N_("_Orders") "/" N_("Transf_orm Terrain"),	"o",
 	orders_menu_callback,	MENU_ORDER_TRANSFORM					},
-  { "/" N_("Orders") "/" N_("Build _Fortress"),		"f",
+  { "/" N_("_Orders") "/" N_("Build _Fortress"),	"f",
 	orders_menu_callback,	MENU_ORDER_FORTRESS					},
-  { "/" N_("Orders") "/" N_("Build Airbas_e"),		"e",
+  { "/" N_("_Orders") "/" N_("Build Airbas_e"),		"e",
 	orders_menu_callback,	MENU_ORDER_AIRBASE					},
-  { "/" N_("Orders") "/" N_("Clean _Pollution"),	"p",
+  { "/" N_("_Orders") "/" N_("Clean _Pollution"),	"p",
 	orders_menu_callback,	MENU_ORDER_POLLUTION					},
-  { "/" N_("Orders") "/" N_("Clean _Nuclear Fallout"),	"n",
+  { "/" N_("_Orders") "/" N_("Clean _Nuclear Fallout"),	"n",
 	orders_menu_callback,	MENU_ORDER_FALLOUT					},
-  { "/" N_("Orders") "/sep1",			NULL,
+  { "/" N_("_Orders") "/sep1",				NULL,
 	NULL,			0,					"<Separator>"	},
-  { "/" N_("Orders") "/" N_("_Sentry"),			"s",
+  { "/" N_("_Orders") "/" N_("_Sentry"),		"s",
 	orders_menu_callback,	MENU_ORDER_SENTRY					},
-  { "/" N_("Orders") "/" N_("Pillage"),		        "<shift>p",
+  { "/" N_("_Orders") "/" N_("Pillage"),		"<shift>p",
 	orders_menu_callback,	MENU_ORDER_PILLAGE					},
-  { "/" N_("Orders") "/sep2",				NULL,
+  { "/" N_("_Orders") "/sep2",				NULL,
 	NULL,			0,					"<Separator>"	},
-  { "/" N_("Orders") "/" N_("Make _Homecity"),		"h",
+  { "/" N_("_Orders") "/" N_("Make _Homecity"),		"h",
 	orders_menu_callback,	MENU_ORDER_HOMECITY					},
-  { "/" N_("Orders") "/" N_("_Unload"),			"u",
-	orders_menu_callback,	MENU_ORDER_UNLOAD					},
-  { "/" N_("Orders") "/" N_("Wake up o_thers"),		"<shift>w",
+  { "/" N_("_Orders") "/" N_("_Unload Transporter"),	"<shift>u",
+	orders_menu_callback,	MENU_ORDER_UNLOAD_TRANSPORTER					},
+  { "/" N_("_Orders") "/" N_("Load"),			"l",
+    orders_menu_callback, MENU_ORDER_LOAD},
+  { "/" N_("_Orders") "/" N_("Unload"), 		"u",
+    orders_menu_callback, MENU_ORDER_UNLOAD},
+  { "/" N_("_Orders") "/" N_("Wake up o_thers"),		"<shift>w",
 	orders_menu_callback,	MENU_ORDER_WAKEUP_OTHERS				},
-  { "/" N_("Orders") "/sep3",				NULL,
+  { "/" N_("_Orders") "/sep3",				NULL,
 	NULL,			0,					"<Separator>"	},
-  { "/" N_("Orders") "/" N_("_Auto Settler"),		"a",
+  { "/" N_("_Orders") "/" N_("_Auto Settler"),		"a",
 	orders_menu_callback,	MENU_ORDER_AUTO_SETTLER					},
-  { "/" N_("Orders") "/" N_("Auto E_xplore"),		"x",
+  { "/" N_("_Orders") "/" N_("Auto E_xplore"),		"x",
 	orders_menu_callback,	MENU_ORDER_AUTO_EXPLORE					},
-  { "/" N_("Orders") "/" N_("_Connect"),		"<shift>c",
-	orders_menu_callback,	MENU_ORDER_CONNECT					},
-  { "/" N_("Orders") "/" N_("Patrol (_Q)"),		"q",
+  { "/" N_("_Orders") "/" N_("_Connect") "/" N_("_Road"), "<ctrl><shift>r",
+   orders_menu_callback, MENU_ORDER_CONNECT_ROAD},
+  { "/" N_("_Orders") "/" N_("_Connect") "/" N_("Rai_l"), "<ctrl><shift>l",
+   orders_menu_callback, MENU_ORDER_CONNECT_RAIL},
+  { "/" N_("_Orders") "/" N_("_Connect") "/" N_("_Irrigate"), "<ctrl><shift>i",
+   orders_menu_callback, MENU_ORDER_CONNECT_IRRIGATE},
+  { "/" N_("_Orders") "/" N_("Patrol (_Q)"),		"q",
 	orders_menu_callback,	MENU_ORDER_PATROL					},
-  { "/" N_("Orders") "/" N_("_Go to"),			"g",
+  { "/" N_("_Orders") "/" N_("_Go to"),			"g",
 	orders_menu_callback,	MENU_ORDER_GOTO						},
-  { "/" N_("Orders") "/" N_("Go|Airlift to City"),	"l",
+  { "/" N_("_Orders") "/" N_("Go|Airlift to City"),	"<shift>l",
 	orders_menu_callback,	MENU_ORDER_GOTO_CITY					},
-  { "/" N_("Orders") "/sep4",				NULL,
+  { "/" N_("_Orders") "/" N_("Return to nearest city"),	"<shift>g",
+	orders_menu_callback,	MENU_ORDER_RETURN },
+  { "/" N_("_Orders") "/sep4",				NULL,
 	NULL,			0,					"<Separator>"	},
-  { "/" N_("Orders") "/" N_("Disband Unit"),		"<shift>d",
+  { "/" N_("_Orders") "/" N_("_Disband Unit"),		"<shift>d",
 	orders_menu_callback,	MENU_ORDER_DISBAND					},
-  { "/" N_("Orders") "/" N_("Diplomat|Spy Actions"),	"d",
+  { "/" N_("_Orders") "/" N_("Diplomat|Spy Actions"),	"d",
 	orders_menu_callback,	MENU_ORDER_DIPLOMAT_DLG					},
-  { "/" N_("Orders") "/" N_("Explode Nuclear"),        "<shift>n",
+  { "/" N_("_Orders") "/" N_("Explode Nuclear"),        "<shift>n",
 	orders_menu_callback,	MENU_ORDER_NUKE						},
-  { "/" N_("Orders") "/sep5",				NULL,
+  { "/" N_("_Orders") "/sep5",				NULL,
 	NULL,			0,					"<Separator>"	},
-  { "/" N_("Orders") "/" N_("_Wait"),			"w",
+  { "/" N_("_Orders") "/" N_("_Wait"),			"w",
 	orders_menu_callback,	MENU_ORDER_WAIT						},
-  { "/" N_("Orders") "/" N_("Done"),			"space",
+  { "/" N_("_Orders") "/" N_("Done"),			"space",
 	orders_menu_callback,	MENU_ORDER_DONE						},
   /* Reports menu ... */
   { "/" N_("_Reports"),					NULL,
 	NULL,			0,					"<Branch>"	},
-  { "/" N_("Reports") "/tearoff1",			NULL,
+  { "/" N_("_Reports") "/tearoff1",			NULL,
 	NULL,			0,					"<Tearoff>"	},
-  { "/" N_("Reports") "/" N_("_Cities"),		"F1",
+  { "/" N_("_Reports") "/" N_("_Cities"),		"F1",
 	reports_menu_callback,	MENU_REPORT_CITIES					},
-  { "/" N_("Reports") "/" N_("_Units"),			"F2",
+  { "/" N_("_Reports") "/" N_("_Units"),		"F2",
 	reports_menu_callback,	MENU_REPORT_UNITS					},
-  { "/" N_("Reports") "/" N_("_Players"),		"F3",
+  { "/" N_("_Reports") "/" N_("_Players"),		"F3",
 	reports_menu_callback,	MENU_REPORT_PLAYERS					},
-  { "/" N_("Reports") "/" N_("_Economy"),		"F5",
+  { "/" N_("_Reports") "/" N_("_Economy"),		"F5",
 	reports_menu_callback,	MENU_REPORT_ECONOMY					},
-  { "/" N_("Reports") "/" N_("_Science"),		"F6",
+  { "/" N_("_Reports") "/" N_("_Science"),		"F6",
 	reports_menu_callback,	MENU_REPORT_SCIENCE					},
-  { "/" N_("Reports") "/sep1",				NULL,
+  { "/" N_("_Reports") "/sep1",				NULL,
 	NULL,			0,					"<Separator>"	},
-  { "/" N_("Reports") "/" N_("_Wonders of the World"),	"F7",
+  { "/" N_("_Reports") "/" N_("_Wonders of the World"),	"F7",
 	reports_menu_callback,	MENU_REPORT_WOW						},
-  { "/" N_("Reports") "/" N_("_Top Five Cities"),	"F8",
+  { "/" N_("_Reports") "/" N_("_Top Five Cities"),	"F8",
 	reports_menu_callback,	MENU_REPORT_TOP_CITIES					},
-  { "/" N_("Reports") "/" N_("_Messages"),		"F10",
+  { "/" N_("_Reports") "/" N_("_Messages"),		"F10",
 	reports_menu_callback,	MENU_REPORT_MESSAGES					},
-  { "/" N_("Reports") "/" N_("_Demographics"),		"F11",
+  { "/" N_("_Reports") "/" N_("_Demographics"),		"F11",
 	reports_menu_callback,	MENU_REPORT_DEMOGRAPHIC					},
-  { "/" N_("Reports") "/" N_("S_paceship"),		"F12",
+  { "/" N_("_Reports") "/" N_("S_paceship"),		"F12",
 	reports_menu_callback,	MENU_REPORT_SPACESHIP					},
   /* Help menu ... */
   { "/" N_("_Help"),					NULL,
 	NULL,			0,					"<LastBranch>"	},
-  { "/" N_("Help") "/tearoff1",				NULL,
+  { "/" N_("_Help") "/tearoff1",			NULL,
 	NULL,			0,					"<Tearoff>"	},
-  { "/" N_("Help") "/" N_("Language_s"),		NULL,
+  { "/" N_("_Help") "/" N_("Language_s"),		NULL,
 	help_menu_callback,	MENU_HELP_LANGUAGES					},
-  { "/" N_("Help") "/" N_("Co_nnecting"),		NULL,
+  { "/" N_("_Help") "/" N_("Co_nnecting"),		NULL,
 	help_menu_callback,	MENU_HELP_CONNECTING					},
-  { "/" N_("Help") "/" N_("C_ontrols"),			NULL,
+  { "/" N_("_Help") "/" N_("C_ontrols"),		NULL,
 	help_menu_callback,	MENU_HELP_CONTROLS					},
-  { "/" N_("Help") "/" N_("C_hatline"),			NULL,
+  { "/" N_("_Help") "/" N_("C_hatline"),		NULL,
 	help_menu_callback,	MENU_HELP_CHATLINE					},
-  { "/" N_("Help") "/" N_("_Worklist Editor"),			NULL,
+  { "/" N_("_Help") "/" N_("_Worklist Editor"),		NULL,
 	help_menu_callback,	MENU_HELP_WORKLIST_EDITOR				},
-  { "/" N_("Help") "/" N_("Citizen _Management"),			NULL,
+  { "/" N_("_Help") "/" N_("Citizen _Management"),	NULL,
 	help_menu_callback,	MENU_HELP_CMA						},
-  { "/" N_("Help") "/" N_("_Playing"),			NULL,
+  { "/" N_("_Help") "/" N_("_Playing"),			NULL,
 	help_menu_callback,	MENU_HELP_PLAYING					},
-  { "/" N_("Help") "/sep1",				NULL,
+  { "/" N_("_Help") "/sep1",				NULL,
 	NULL,			0,					"<Separator>"	},
-  { "/" N_("Help") "/" N_("City _Improvements"),        NULL,
+  { "/" N_("_Help") "/" N_("City _Improvements"),	NULL,
 	help_menu_callback,	MENU_HELP_IMPROVEMENTS					},
-  { "/" N_("Help") "/" N_("_Units"),			NULL,
+  { "/" N_("_Help") "/" N_("_Units"),			NULL,
 	help_menu_callback,	MENU_HELP_UNITS						},
-  { "/" N_("Help") "/" N_("Com_bat"),			NULL,
+  { "/" N_("_Help") "/" N_("Com_bat"),			NULL,
 	help_menu_callback,	MENU_HELP_COMBAT					},
-  { "/" N_("Help") "/" N_("_ZOC"),			NULL,
+  { "/" N_("_Help") "/" N_("_ZOC"),			NULL,
 	help_menu_callback,	MENU_HELP_ZOC						},
-  { "/" N_("Help") "/" N_("Techno_logy"),		NULL,
+  { "/" N_("_Help") "/" N_("Techno_logy"),		NULL,
 	help_menu_callback,	MENU_HELP_TECH						},
-  { "/" N_("Help") "/" N_("_Terrain"),			NULL,
+  { "/" N_("_Help") "/" N_("_Terrain"),			NULL,
 	help_menu_callback,	MENU_HELP_TERRAIN					},
-  { "/" N_("Help") "/" N_("Won_ders"),			NULL,
+  { "/" N_("_Help") "/" N_("Won_ders"),			NULL,
 	help_menu_callback,	MENU_HELP_WONDERS					},
-  { "/" N_("Help") "/" N_("_Government"),		NULL,
+  { "/" N_("_Help") "/" N_("_Government"),		NULL,
 	help_menu_callback,	MENU_HELP_GOVERNMENT					},
-  { "/" N_("Help") "/" N_("Happin_ess"),		NULL,
+  { "/" N_("_Help") "/" N_("Happin_ess"),		NULL,
 	help_menu_callback,	MENU_HELP_HAPPINESS					},
-  { "/" N_("Help") "/" N_("Space _Race"),		NULL,
+  { "/" N_("_Help") "/" N_("Space _Race"),		NULL,
 	help_menu_callback,	MENU_HELP_SPACE_RACE					},
-  { "/" N_("Help") "/sep2",				NULL,
+  { "/" N_("_Help") "/sep2",				NULL,
 	NULL,			0,					"<Separator>"	},
-  { "/" N_("Help") "/" N_("_Copying"),			NULL,
+  { "/" N_("_Help") "/" N_("_Copying"),			NULL,
 	help_menu_callback,	MENU_HELP_COPYING					},
-  { "/" N_("Help") "/" N_("_About"),			NULL,
+  { "/" N_("_Help") "/" N_("_About"),			NULL,
 	help_menu_callback,	MENU_HELP_ABOUT						}
 };
 
@@ -783,7 +842,7 @@ static GtkItemFactoryEntry menu_items[]	=
 
   Path should include underscores like in the menu itself.
 *****************************************************************/
-static const char *translate_menu_path(const char *path, int remove_uline)
+static const char *translate_menu_path(const char *path, bool remove_uline)
 {
 #ifndef ENABLE_NLS
   static char res[100];
@@ -849,8 +908,9 @@ void setup_menus(GtkWidget *window, GtkWidget **menubar)
 
   item_factory=gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", accel);
    
-  for(i=0; i<nmenu_items; i++) {
-    menu_items[i].path = mystrdup(translate_menu_path(menu_items[i].path, 0));
+  for (i = 0; i < nmenu_items; i++) {
+    menu_items[i].path =
+	mystrdup(translate_menu_path(menu_items[i].path, FALSE));
   }
   
   gtk_item_factory_create_items(item_factory, nmenu_items, menu_items, NULL);
@@ -872,7 +932,7 @@ static void menus_set_sensitive(const char *path, int sensitive)
 {
   GtkWidget *item;
 
-  path = translate_menu_path(path, 1);
+  path = translate_menu_path(path, TRUE);
   
   if(!(item=gtk_item_factory_get_widget(item_factory, path))) {
     freelog(LOG_ERROR,
@@ -892,7 +952,7 @@ static void menus_set_active(const char *path, int active)
 {
   GtkWidget *item;
 
-  path = translate_menu_path(path, 1);
+  path = translate_menu_path(path, TRUE);
 
   if (!(item = gtk_item_factory_get_widget(item_factory, path))) {
     freelog(LOG_ERROR,
@@ -913,7 +973,7 @@ static void menus_set_shown(const char *path, int shown)
 {
   GtkWidget *item;
   
-  path = translate_menu_path(path, 1);
+  path = translate_menu_path(path, TRUE);
   
   if(!(item=gtk_item_factory_get_widget(item_factory, path))) {
     freelog(LOG_ERROR, "Can't show non-existent menu %s.", path);
@@ -937,7 +997,7 @@ static void menus_rename(const char *path, char *s)
 {
   GtkWidget *item;
   
-  path = translate_menu_path(path, 1);
+  path = translate_menu_path(path, TRUE);
   
   if(!(item=gtk_item_factory_get_widget(item_factory, path))) {
     freelog(LOG_ERROR, "Can't rename non-existent menu %s.", path);
@@ -952,6 +1012,14 @@ static void menus_rename(const char *path, char *s)
 }
 
 
+/****************************************************************
+  The player has chosen a government from the menu.
+*****************************************************************/
+static void government_callback(GtkMenuItem *item, gpointer data)
+{
+  set_government_choice(GPOINTER_TO_INT(data));
+}
+
 /****************************************************************
 Note: the menu strings should contain underscores as in the
 menu_items struct. The underscores will be removed elsewhere if
@@ -959,28 +1027,72 @@ the string is used for a lookup via gtk_item_factory_get_widget()
 *****************************************************************/
 void update_menus(void)
 {
-  if(get_client_state()!=CLIENT_GAME_RUNNING_STATE) {
+  if (!can_client_change_view()) {
     menus_set_sensitive("<main>/_Reports", FALSE);
-    menus_set_sensitive("<main>/_Kingdom", FALSE);
+    menus_set_sensitive("<main>/Gov_ernment", FALSE);
     menus_set_sensitive("<main>/_View", FALSE);
     menus_set_sensitive("<main>/_Orders", FALSE);
   } else {
     struct unit *punit;
-    menus_set_sensitive("<main>/_Reports", TRUE);
-    menus_set_sensitive("<main>/_Kingdom", TRUE);
-    menus_set_sensitive("<main>/_View", TRUE);
-    menus_set_sensitive("<main>/_Orders", !client_is_observer());
+    GtkWidget *item;
+    const char *path =
+	translate_menu_path("<main>/Gov_ernment/_Change Government", TRUE);
+    GtkWidget *parent = gtk_item_factory_get_widget(item_factory, path);
 
-    menus_set_sensitive("<main>/_Kingdom/_Tax Rates", !client_is_observer());
-    menus_set_sensitive("<main>/_Kingdom/Work_lists", !client_is_observer());
-    menus_set_sensitive("<main>/_Kingdom/_Revolution",
-			!client_is_observer());
+    if (parent) {
+      int i;
+      GList *iter, *iter_next;
+
+      /* remove previous government entries. */
+      iter = gtk_container_children(GTK_CONTAINER(parent));
+      for (iter = g_list_nth(iter, 2); iter; iter = iter_next) {
+        iter_next = iter->next;
+
+        gtk_container_remove(GTK_CONTAINER(parent), GTK_WIDGET(iter->data));
+      }
+
+      /* add new government entries. */
+      for (i = 0; i < game.government_count; i++) {
+        struct government *g = &governments[i];
+
+        if (i != game.government_when_anarchy) {
+          item = gtk_menu_item_new_with_label(g->name);
+          gtk_widget_show(item);
+
+          gtk_signal_connect(GTK_OBJECT(item), "activate",
+            GTK_SIGNAL_FUNC(government_callback), GINT_TO_POINTER(g->index));
+
+          if (!can_change_to_government(game.player_ptr, i)) {
+            gtk_widget_set_sensitive(item, FALSE);
+	  }
+
+          gtk_menu_shell_append(GTK_MENU_SHELL(parent), item);
+          gtk_widget_show(item);
+        }
+      }
+    }
+
+    menus_set_sensitive("<main>/_Reports", TRUE);
+    menus_set_sensitive("<main>/Gov_ernment", TRUE);
+    menus_set_sensitive("<main>/_View", TRUE);
+    menus_set_sensitive("<main>/_Orders", can_client_issue_orders());
+
+    menus_set_sensitive("<main>/Gov_ernment/_Tax Rates",
+			can_client_issue_orders());
+    menus_set_sensitive("<main>/Gov_ernment/Work_lists",
+			can_client_issue_orders());
+    menus_set_sensitive("<main>/Gov_ernment/_Change Government",
+			can_client_issue_orders());
 
     menus_set_sensitive("<main>/_Reports/S_paceship",
 			(game.player_ptr->spaceship.state!=SSHIP_NONE));
 
     menus_set_active("<main>/_View/Map _Grid", draw_map_grid);
+    menus_set_sensitive("<main>/_View/National _Borders", game.borders > 0);
+    menus_set_active("<main>/_View/National _Borders", draw_borders);
     menus_set_active("<main>/_View/City _Names", draw_city_names);
+    menus_set_sensitive("<main>/_View/City G_rowth", draw_city_names);
+    menus_set_active("<main>/_View/City G_rowth", draw_city_growth);
     menus_set_active("<main>/_View/City _Productions", draw_city_productions);
     menus_set_active("<main>/_View/Terrain", draw_terrain);
     menus_set_active("<main>/_View/Coastline", draw_coastline);
@@ -999,7 +1111,7 @@ void update_menus(void)
 
     /* Remaining part of this function: Update Orders menu */
 
-    if (client_is_observer()) {
+    if (!can_client_issue_orders()) {
       return;
     }
 
@@ -1009,7 +1121,7 @@ void update_menus(void)
       char *transfmt = _("Transf_orm to %s");
       char irrtext[128], mintext[128], transtext[128];
       char *roadtext;
-      enum tile_terrain_type  ttype;
+      Terrain_type_id  ttype;
       struct tile_type *      tinfo;
 
       sz_strlcpy(irrtext, _("Build _Irrigation"));
@@ -1047,29 +1159,41 @@ void update_menus(void)
 			  can_unit_do_activity(punit, ACTIVITY_PILLAGE));
       menus_set_sensitive("<main>/_Orders/Make _Homecity",
 			  can_unit_change_homecity(punit));
-      menus_set_sensitive("<main>/_Orders/_Unload",
-			  get_transporter_capacity(punit)>0);
+      menus_set_sensitive("<main>/_Orders/_Unload Transporter",
+			  get_transporter_occupancy(punit) > 0);
+      menus_set_sensitive("<main>/_Orders/Load",
+	can_unit_load(punit, find_transporter_for_unit(punit,
+						       punit->tile)));
+      menus_set_sensitive("<main>/_Orders/Unload",
+	(can_unit_unload(punit, find_unit_by_id(punit->transported_by))
+	 && can_unit_exist_at_tile(punit, punit->tile)));
       menus_set_sensitive("<main>/_Orders/Wake up o_thers", 
 			  is_unit_activity_on_tile(ACTIVITY_SENTRY,
-                                                   punit->x, punit->y));
+                                                   punit->tile));
       menus_set_sensitive("<main>/_Orders/_Auto Settler",
                           can_unit_do_auto(punit));
       menus_set_sensitive("<main>/_Orders/Auto E_xplore",
                           can_unit_do_activity(punit, ACTIVITY_EXPLORE));
-      menus_set_sensitive("<main>/_Orders/_Connect",
-                          can_unit_do_connect(punit, ACTIVITY_IDLE));
-      menus_set_sensitive("<main>/_Orders/Patrol (_Q)",
-                          can_unit_do_activity(punit, ACTIVITY_PATROL));
+      menus_set_sensitive("<main>/_Orders/_Connect/_Road",
+                          can_unit_do_connect(punit, ACTIVITY_ROAD));
+      menus_set_sensitive("<main>/_Orders/_Connect/Rai_l",
+                          can_unit_do_connect(punit, ACTIVITY_RAILROAD));
+      menus_set_sensitive("<main>/_Orders/_Connect/_Irrigate",
+                          can_unit_do_connect(punit, ACTIVITY_IRRIGATE));
+      menus_set_sensitive("<main>/_Orders/Return to nearest city",
+			  !(is_air_unit(punit) || is_heli_unit(punit)));
+      menus_set_sensitive("<main>/_Orders/_Disband Unit",
+                          !unit_flag(punit, F_UNDISBANDABLE));
       menus_set_sensitive("<main>/_Orders/Diplomat|Spy Actions",
                           (is_diplomat_unit(punit)
                            && diplomat_can_do_action(punit, DIPLOMAT_ANY_ACTION,
-						     punit->x, punit->y)));
+						     punit->tile)));
       menus_set_sensitive("<main>/_Orders/Explode Nuclear",
 			  unit_flag(punit, F_NUCLEAR));
       if (unit_flag(punit, F_HELP_WONDER))
 	menus_rename("<main>/_Orders/_Build City", _("Help _Build Wonder"));
       else if (unit_flag(punit, F_CITIES)) {
-	if (map_get_city(punit->x, punit->y))
+	if (map_get_city(punit->tile))
 	  menus_rename("<main>/_Orders/_Build City", _("Add to City (_B)"));
 	else
 	  menus_rename("<main>/_Orders/_Build City", _("_Build City"));
@@ -1080,7 +1204,7 @@ void update_menus(void)
       if (unit_flag(punit, F_TRADE_ROUTE))
 	menus_rename("<main>/_Orders/Build _Road", _("Make Trade _Route"));
       else if (unit_flag(punit, F_SETTLERS)) {
-	if (map_has_special(punit->x, punit->y, S_ROAD)) {
+	if (map_has_special(punit->tile, S_ROAD)) {
 	  roadtext = _("Build _Railroad");
 	  road_activity=ACTIVITY_RAILROAD;  
 	} 
@@ -1089,32 +1213,31 @@ void update_menus(void)
 	  road_activity=ACTIVITY_ROAD;  
 	}
 	menus_rename("<main>/_Orders/Build _Road", roadtext);
-      }
-      else
+      } else {
 	menus_rename("<main>/_Orders/Build _Road", _("Build _Road"));
+      }
 
-      ttype = map_get_tile(punit->x, punit->y)->terrain;
+      ttype = punit->tile->terrain;
       tinfo = get_tile_type(ttype);
-      if ((tinfo->irrigation_result != T_LAST) && (tinfo->irrigation_result != ttype))
-	{
-	  my_snprintf (irrtext, sizeof(irrtext), irrfmt,
-		   (get_tile_type(tinfo->irrigation_result))->terrain_name);
-	}
-      else if (map_has_special(punit->x, punit->y, S_IRRIGATION) &&
-	       player_knows_techs_with_flag(game.player_ptr, TF_FARMLAND))
-	{
-	  sz_strlcpy (irrtext, _("Bu_ild Farmland"));
-	}
-      if ((tinfo->mining_result != T_LAST) && (tinfo->mining_result != ttype))
-	{
-	  my_snprintf (mintext, sizeof(mintext), minfmt,
-		   (get_tile_type(tinfo->mining_result))->terrain_name);
-	}
-      if ((tinfo->transform_result != T_LAST) && (tinfo->transform_result != ttype))
-	{
-	  my_snprintf (transtext, sizeof(transtext), transfmt,
-		   (get_tile_type(tinfo->transform_result))->terrain_name);
-	}
+      if (tinfo->irrigation_result != T_NONE
+	  && tinfo->irrigation_result != ttype) {
+	my_snprintf(irrtext, sizeof(irrtext), irrfmt,
+		    (get_tile_type(tinfo->irrigation_result))->terrain_name);
+      } else if (map_has_special(punit->tile, S_IRRIGATION)
+		 && player_knows_techs_with_flag(game.player_ptr,
+						 TF_FARMLAND)) {
+	sz_strlcpy(irrtext, _("Bu_ild Farmland"));
+      }
+      if (tinfo->mining_result != T_NONE
+	  && tinfo->mining_result != ttype) {
+	my_snprintf(mintext, sizeof(mintext), minfmt,
+		    (get_tile_type(tinfo->mining_result))->terrain_name);
+      }
+      if (tinfo->transform_result != T_NONE
+	  && tinfo->transform_result != ttype) {
+	my_snprintf(transtext, sizeof(transtext), transfmt,
+		    (get_tile_type(tinfo->transform_result))->terrain_name);
+      }
 
       menus_rename("<main>/_Orders/Build _Irrigation", irrtext);
       menus_rename("<main>/_Orders/Build _Mine", mintext);
