@@ -24,6 +24,17 @@ struct Sprite {
   int has_mask;
 };
 
+struct canvas {
+  Pixmap pixmap;
+};
+
 Pixmap create_overlay_unit(int i);
 
+extern struct Sprite *intro_gfx_sprite;
+extern struct Sprite *radar_gfx_sprite;
+extern Cursor         goto_cursor;
+extern Cursor         drop_cursor;
+extern Cursor         nuke_cursor;
+extern Cursor         patrol_cursor;
+
 #endif  /* FC__GRAPHICS_H */
