@@ -13,8 +13,10 @@
 #ifndef FC__GRAPHICS_G_H
 #define FC__GRAPHICS_G_H
 
-int isometric_view_supported(void);
-int overhead_view_supported(void);
+#include "shared.h"		/* bool type */
+
+bool isometric_view_supported(void);
+bool overhead_view_supported(void);
 
 void load_intro_gfx(void);
 void load_cursors(void);
@@ -23,11 +25,14 @@ void free_intro_radar_sprites(void);
 
 struct Sprite;			/* opaque type, real type is gui-dep */
 
-char **gfx_fileextensions(void);
+const char **gfx_fileextensions(void);
 
 struct Sprite *load_gfxfile(const char *filename);
 struct Sprite *crop_sprite(struct Sprite *source,
-			   int x, int y, int width, int height);
+			   int x, int y, int width, int height,
+			   struct Sprite *mask,
+			   int mask_offset_x, int mask_offset_y);
+void get_sprite_dimensions(struct Sprite *sprite, int *width, int *height);
 void free_sprite(struct Sprite *s);
 
 #endif  /* FC__GRAPHICS_G_H */
