@@ -15,10 +15,11 @@
 
 #include "shared.h"		/* bool type */
 
-struct unit;
+#include "fc_types.h"
 
 void ui_init(void);
 void ui_main(int argc, char *argv[]);
+void update_conn_list_dialog(void);
 void sound_bell(void);
 void add_net_input(int);
 void remove_net_input(void);
